/*
*   InitCameraGui
*
*   Step of the calibration which check if the camera is availibe and initialize it.
*   The initialization is done in another thread to not block the application.
*/

#include "initcameragui.h"
#include "ui_initcameragui.h"



CameraStartThread::CameraStartThread(Camera *c) : QThread() {
    camera = c;
}

void CameraStartThread::run() {

    int err = camera->start();
    emit setupReady(err);
}




InitCameraGui::InitCameraGui(SandboxSetup *_setup, QWidget *parent) :
    SubApp("Initialize configuration", "Error", parent),
    ui(new Ui::InitCameraGui)
{
    setup = _setup;
    ui->setupUi(this);
    ui->label->setText(QString("Initializing device..."));

    workerThread = new CameraStartThread(setup->getCamera());
    connect(workerThread, &CameraStartThread::setupReady, this, &InitCameraGui::setupReady);
    //connect(workerThread, &QThread::finished, workerThread, &QObject::deleteLater); // delete thread when finished
}

InitCameraGui::~InitCameraGui()
{
    workerThread->quit();
    workerThread->wait();
    delete workerThread;
    delete ui;
}

void InitCameraGui::showEvent(QShowEvent *event){

    QWidget::showEvent(event);
    QTimer::singleShot(10, this, &InitCameraGui::setupCamera);
}

void InitCameraGui::setupCamera(){

    workerThread->start();
}

void InitCameraGui::setupReady(int err){
    if(err){
       QString msg = "No device found";
       ui->label->setText(msg);
    }

    emit sendNotif(err);
}


