/*
*   Sandbox
*
*   Interface for the devs to create other setup applications
*/

#include "../../inc/sandboxSetup.h"


SandboxSetup::SandboxSetup(){
    projection = new Projection();
    camera = new Camera();
    beamer = new Beamer();
}

SandboxSetup::~SandboxSetup(){
    delete beamer;
    delete camera;
    delete projection;
}


//
//  PUBLIC
//


// return 1 when config can't be saved in file
int SandboxSetup::saveConfig(char *path){
    if(SandboxConfig::saveAdjustingMatrixInto(path, projection->getAdjustingMatrix(), projection->getAngleRotation()))
        return 1;
    
    if(SandboxConfig::saveDistanceToSandboxTopInto(path, projection->getDistanceTopSandbox()))
        return 1;

    if(SandboxConfig::saveCroppingMaskInto(path, camera->getCroppingMask()))
        return 1;

    if(SandboxConfig::saveBeamerResolutionInto(path, cv::Size(beamer->getWidth(), beamer->getHeight())))
        return 1;

    if(SandboxConfig::saveBeamerPositionInto(path, beamer->getPosition()))
        return 1;

    if(SandboxConfig::saveFrameProcessProfilInto(path, *beamer->getProfil()))
        return 1;

    return 0;
}


int SandboxSetup::saveConfig(){
    return saveConfig(defaultConfigFilePath);
}

int SandboxSetup::loadCroppingMaskAndAdjustingMatrix(char *path){
    int err = SandboxConfig::loadAdjustingMatrixFrom(path, projection);
    if(err){ return err; }
    err = SandboxConfig::loadCroppingMaskFrom(path, camera);
    return err;
}

int SandboxSetup::loadCroppingMaskAndAdjustingMatrix(){
    return loadCroppingMaskAndAdjustingMatrix(defaultConfigFilePath);
}

int SandboxSetup::loadFrameProcessProfil(char *path){
    return SandboxConfig::loadFrameProcessProfilFrom(path, beamer->getProfil());
}

int SandboxSetup::loadFrameProcessProfil(){
    return loadFrameProcessProfil(defaultConfigFilePath);
}


/*  Assuming points positions are :
 *      pts[0] : top left
 *      pts[1] : bottom left
 *      pts[2] : bottom right
 *      pts[3] : top right
 * 
 *  center : center of the rotation in the projected frame
 */
// TODO : retourner la valeur sans set les variables
void SandboxSetup::setupAdjustMatrix(std::vector<cv::Point2i> rectPoints, cv::Point2i center){

    // Set adjusting matrix for the projection
    double angle = getAngle(rectPoints);
    cv::Mat_<float> matRotation = cv::getRotationMatrix2D(center, angle, 1);
    projection->setAdjustingMatrix(matRotation);
    projection->setAngleRotation(angle);
}


/*  Assuming points positions are :
 *      pts[0] : top left
 *      pts[1] : bottom left
 *      pts[2] : bottom right
 *      pts[3] : top right
 */
// TODO : retourner la valeur sans set les variables
void SandboxSetup::setupCroppingMask(std::vector<cv::Point2i> rectPoints, cv::Point2i center){

    // Set cropping mask
    double angle = getAngle(rectPoints);
    cv::Point2i ptl = projection->rotatePixel(center, angle, rectPoints[0]);
    cv::Point2i pbl = projection->rotatePixel(center, angle, rectPoints[1]);
    cv::Point2i pbr = projection->rotatePixel(center, angle, rectPoints[2]);
    cv::Point2i ptr = projection->rotatePixel(center, angle, rectPoints[3]);
    cv::Point2i pbot = (pbl.y < pbr.y) ? pbl : pbr;
    int widthTop = ptr.x - ptl.x;

    cv::Size rectSize = cv::Size(widthTop, pbot.y - rectPoints[0].y);
    camera->setCroppingMask(cv::Rect(ptl, rectSize)); // croppingMask
}



//
//  PRIVATE
//


double SandboxSetup::getAngle(std::vector<cv::Point2i> rectPoints){
    int widthTop = rectPoints[3].x - rectPoints[0].x;
    return toDegrees(atan((double)(rectPoints[3].y - rectPoints[0].y) / widthTop));
}

double SandboxSetup::toDegrees(double radians){
    return radians * (180.0 / M_PI);
}
